// niz4.h - Genericka klasa nizova.

#ifndef _niz4_h_
#define _niz4_h_

#include <iostream>
using namespace std;

class GIndeks {};          // KLASA GRESAKA: Nedozvoljen indeks.
inline ostream& operator<< (ostream& d, const GIndeks&)
  { return d << "*** Nedozvoljen indeks!"; }

class GPrazan {};          // KLASA GRESAKA: Niz je prazan.
inline ostream& operator<< (ostream& d, const GPrazan&)
  { return d << "*** Niz je prazan!"; }

template <typename T>      // GENERICKA KLASA NIZOVA:
  class Niz {
    T* niz;                                     // Elementi niza.
    int kap, duz;                               // Kapacitet i duzina niza.
    void kopiraj (const Niz& n);                // Kopiranje u objekat.
  public:
    explicit Niz (int k=10)                     // Stvaranje praznog niza.
      { niz = new T [kap = k]; duz = 0; }
    Niz (const Niz& n) { kopiraj (n); }         // Konstruktor kopije.
    ~Niz () { delete [] niz; }                  // Destruktor.
    Niz& operator= (const Niz& n) {             // Dodela vrednosti.
      if (this != &n) { delete [] niz; kopiraj (n); }
      return *this;
    }
    int vel () const { return duz; }            // Duzina niza.
    Niz& dodaj (const T& t);                    // Dodavanje na kraj.
    T uzmi ();                                  // Uzimanje s kraja.
    T& operator[] (int i) {                     // Dohvatanje elementa
      if (i<0 || i>=duz) throw GIndeks ();      // - promenljivog niza,
      return niz[i];
    }
    const T& operator[] (int i) const {         // - nepromenljivog niza.
      if (i<0 || i>=duz) throw GIndeks ();
      return niz[i];
    }
    Niz& isprazni () { duz = 0; return *this; } // Praznjenje niza.
  }; // class Niz<T>

template <typename T>      // DEFINICIJE METODA KLASE NIZOVA:
  void Niz<T>::kopiraj (const Niz& n) {         // Kopiranje u objekat.
    niz = new T [kap = n.kap];
    duz = n.duz;
    for (int i=0; i<duz; i++)
      niz[i] = n.niz[i];
  }
 
template <typename T>
  Niz<T>& Niz<T>::dodaj (const T& t) {          // Dodavanje na kraj.
    if (duz == kap) {
      T* pom = new T [kap+=5];
      for (int i=0; i<duz; i++) pom[i] = niz[i];
      delete [] niz; niz = pom;
    }
    niz[duz++] = t;
    return *this;
  }

template <typename T>  
  T Niz<T>::uzmi () {                           // Uzimanje s kraja.
    if (! duz) throw GPrazan ();
    T t = niz[--duz];
    if (kap-duz>10) {
      T* pom = new T [kap-=5];
      for (int i=0; i<duz; i++) pom[i] = niz[i];
      delete [] niz; niz = pom;
    }
    return t;
  }

#endif
